SECRET_KEY = "test"
INSTALLED_PACKAGES = [
    "plain.models",
    "plain.sessions",
]
DATABASES = {
    "default": {
        "ENGINE": "plain.models.backends.sqlite3",
        "NAME": ":memory:",
    }
}
MIDDLEWARE = [
    "plain.sessions.middleware.SessionMiddleware",
]



from plain import models


class SessionManager(models.Manager):
    use_in_migrations = True

    def encode(self, session_dict):
        """
        Return the given session dictionary serialized and encoded as a string.
        """
        session_store_class = self.model.get_session_store_class()
        return session_store_class().encode(session_dict)

    def save(self, session_key, session_dict, expire_date):
        s = self.model(session_key, self.encode(session_dict), expire_date)
        if session_dict:
            s.save()
        else:
            s.delete()  # Clear sessions with no data.
        return s


@models.register_model
class Session(models.Model):
    """
    Plain provides full support for anonymous sessions. The session
    framework lets you store and retrieve arbitrary data on a
    per-site-visitor basis. It stores data on the server side and
    abstracts the sending and receiving of cookies. Cookies contain a
    session ID -- not the data itself.

    The Plain sessions framework is entirely cookie-based. It does
    not fall back to putting session IDs in URLs. This is an intentional
    design decision. Not only does that behavior make URLs ugly, it makes
    your site vulnerable to session-ID theft via the "Referer" header.

    For complete documentation on using Sessions in your code, consult
    the sessions documentation that is shipped with Plain (also available
    on the Plain web site).
    """

    session_key = models.CharField(max_length=40, primary_key=True)
    session_data = models.TextField()
    expire_date = models.DateTimeField(db_index=True)

    objects = SessionManager()

    def __str__(self):
        return self.session_key

    @classmethod
    def get_session_store_class(cls):
        from plain.sessions.backends.db import SessionStore

        return SessionStore

    def get_decoded(self):
        session_store_class = self.get_session_store_class()
        return session_store_class().decode(self.session_data)



# Cookie name. This can be whatever you want.
SESSION_COOKIE_NAME = "sessionid"
# Age of cookie, in seconds (default: 2 weeks).
SESSION_COOKIE_AGE = 60 * 60 * 24 * 7 * 2
# A string like "example.com", or None for standard domain cookie.
SESSION_COOKIE_DOMAIN = None
# Whether the session cookie should be secure (https:// only).
SESSION_COOKIE_SECURE = True
# The path of the session cookie.
SESSION_COOKIE_PATH = "/"
# Whether to use the HttpOnly flag.
SESSION_COOKIE_HTTPONLY = True
# Whether to set the flag restricting cookie leaks on cross-site requests.
# This can be 'Lax', 'Strict', 'None', or False to disable the flag.
SESSION_COOKIE_SAMESITE = "Lax"
# Whether to save the session data on every request.
SESSION_SAVE_EVERY_REQUEST = False
# Whether a user's session cookie expires when the web browser is closed.
SESSION_EXPIRE_AT_BROWSER_CLOSE = False
# The module to store session data
SESSION_CLASS = "plain.sessions.backends.db.SessionStore"



from plain.sessions.models import Session


def test_session_created(db, client):
    assert Session.objects.count() == 0

    response = client.get("/")

    assert response.status_code == 200

    assert Session.objects.count() == 1



import click

from plain.runtime import settings
from plain.utils.module_loading import import_string


@click.group()
def cli():
    """Sessions management commands."""
    pass


@cli.command()
def clear_expired():
    Session = import_string(settings.SESSION_CLASS)
    try:
        Session.clear_expired()
    except NotImplementedError:
        raise NotImplementedError(
            f"Session '{settings.SESSION_CLASS}' doesn't support clearing expired "
            "sessions."
        )



# Generated by Plain 5.0.dev20230814020017 on 2023-08-14 02:09

from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainsessions", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="session",
            options={},
        ),
        migrations.AlterField(
            model_name="session",
            name="expire_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="session",
            name="session_data",
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name="session",
            name="session_key",
            field=models.CharField(max_length=40, primary_key=True),
        ),
    ]



from plain.urls import RouterBase, path, register_router
from plain.views import View


class IndexView(View):
    def get(self):
        # Store something so the session is saved
        self.request.session["foo"] = "bar"
        return "test"


@register_router
class Router(RouterBase):
    namespace = ""
    urls = [
        path("", IndexView),
    ]



import plain.sessions.models
from plain import models
from plain.models import migrations


class Migration(migrations.Migration):
    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Session",
            fields=[
                (
                    "session_key",
                    models.CharField(
                        max_length=40,
                        primary_key=True,
                    ),
                ),
                ("session_data", models.TextField()),
                (
                    "expire_date",
                    models.DateTimeField(db_index=True),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", plain.sessions.models.SessionManager()),
            ],
        ),
    ]
